/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package main;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.IOException;
import javax.swing.*;

import required.ColorFunctions;
import required.CurrencyConverter;
import required.Domain;
import required.Message;
import required.Tools;
import required.Validation;


//********************************************* START CLASS *********************************************//


/**
 * This class builds the currency converter GUI.
 * 
 * @author Alexander Wait
 * @version 2012-05-05
 */
public class CurrencyConverterGui extends JApplet
{
   private static final String copyright = "Copyright (c) 2012 by Alexander Wait - All Rights Reserved";

   private static final long serialVersionUID = 1L;
   
   
   public static boolean isApplication = false; 
   
   public static boolean isVisible = false;
   
   
   private static final String DEFAULT_AMOUNT = "1";
   
   
   private static final int FRAME_SIZE_X = 420;
   
   private static final int FRAME_SIZE_Y = 550;
  

   //************************************** START INITIALISATION CODE **************************************//

   
   /**
    * Browser constructor.
    * ESCA-JAVA0166:
    */
   public void init()
   {
      if (!isApplication) 
      { 
         String copyrightParam = getParameter("copyright");
         
         if ((copyrightParam == null) || !copyrightParam.equals(copyright)) 
         {
            Message.showMessage("Invalid Copyright", "WARNING", "warning"); 
         }       
         
         else if (!getDocumentBase().getHost().equals(Domain.getDomain())) 
         {
            Message.showMessage("Unauthorised Applet Use", "ERROR", "error");
         }
        
         else
         {
            initialiser(); includeResizeEvent(); 
            
            setSize(FRAME_SIZE_X, FRAME_SIZE_Y);
         }
      }
   }
 
   
   
   /**
    * Application constructor.
    */
   public CurrencyConverterGui()
   {
      if (isApplication) 
      {
         initialiser(); 
         
         
         frame = new JFrame(); 
         
         frame.setMinimumSize(new Dimension(FRAME_SIZE_X, FRAME_SIZE_Y));
         
         frame.setSize(FRAME_SIZE_X,FRAME_SIZE_Y);
         
         frame.setLocationRelativeTo(null); 
         
         frame.setTitle("Currency Program");
         
         frame.add(contentPane); 
         
        
         setVisibility(); 
      }
 
   }
   
   
   
   /**
    * GUI initializer.
    * ESCA-JAVA0076:
    */
   private void initialiser()
   {   
      currency = new CurrencyConverter(); validation = new Validation();
      
      
      
      // Initialize content pane and layout.
      
      contentPane = getContentPane(); 
      
      contentPane.setLayout(new GridBagLayout());
      
      
      
      // Initialize text area GUI components.
      
      firstCountryDisplay = new JTextArea(); 
      
      firstCountryDisplay.setEnabled(false);
      
      firstCountryDisplay.setFont(new Font("Dialog", Font.BOLD, 16));
      
      
      secondCountryDisplay = new JTextArea(); 
      
      secondCountryDisplay.setEnabled(false);
      
      secondCountryDisplay.setFont(new Font("Dialog", Font.BOLD, 16));
      
      
      conversionDisplay = new JTextArea(); 
      
      conversionDisplay.setEditable(false);
      
      conversionDisplay.setFont(new Font("Dialog", Font.BOLD, 16));
      
      conversionDisplay.setBorder(BorderFactory.createEtchedBorder());
      
      
      
      // Initialize text field GUI components.
      
      amountField = new JTextField();
      
      amountField.setHorizontalAlignment(JTextField.CENTER);
      
      amountField.setFont(new Font("Serif", Font.BOLD, 16));
      
      
      
      // Initialize combo box GUI components.
      
      firstCurrencyBox = new JComboBox<String>(currency.returnCurrencyName());
      
      firstCurrencyBox.addActionListener(new FirstCurrencyListener());
      
      firstCurrencyBox.setMaximumRowCount(11); 
      
      firstCurrencyBox.setSelectedIndex(1);
      
      firstCurrencyBox.setFocusable(false);
      
      
      secondCurrencyBox = new JComboBox<String>(currency.returnCurrencyName());
      
      secondCurrencyBox.addActionListener(new SecondCurrencyListener());
      
      secondCurrencyBox.setMaximumRowCount(11);
      
      secondCurrencyBox.setSelectedIndex(59);
      
      secondCurrencyBox.setFocusable(false);
      
      
      
      // Initialize the country displays.
      
      firstCountryDisplay.setText
      (
         currency.returnCountryName((String)firstCurrencyBox.getSelectedItem())
      );
      
      
      secondCountryDisplay.setText
      (
         currency.returnCountryName((String)secondCurrencyBox.getSelectedItem())
      );
      
      
      
      // Initialize button GUI components.
      
      run = new JButton(new ImageIcon(this.getClass().getResource
      (
         "/files/images/FinanceToolCalculateNormal.png"))
      );
      
      
      run.setRolloverIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/FinanceToolCalculateRollover.png"))
      );

      
      run.setPressedIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/FinanceToolCalculatePressed.png"))
      );
      
      
      run.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
      
      run.addActionListener(new RunButtonListener());
      
      
      clear = new JButton(new ImageIcon(this.getClass().getResource
      (
         "/files/images/FinanceToolClearNormal.png"))
      );
      
      
      clear.setRolloverIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/FinanceToolClearRollover.png"))
      );

      
      clear.setPressedIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/FinanceToolClearPressed.png"))
      );
      
      
      clear.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
      
      clear.addActionListener(new ClearButtonListener());
      
      
      
      // Initialize JPanel GUI components.
      
      buttonPanel = new JPanel(); 
      
      buttonPanel.setLayout (new FlowLayout(FlowLayout.LEADING));
      
      buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
      
      
      
      // Add GUI components to JPanels.
      
      buttonPanel.add(run); buttonPanel.add(clear);
      
      
      
      // Method calls for GUI initialization.

      gridBagConstraints(); setColors(); setConvertAmount(DEFAULT_AMOUNT);
   }
   
   
   
   /**
    * Sets the background color.
    */
   private void setColors()
   {
      contentPane.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      buttonPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      firstCountryDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      secondCountryDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      amountField.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      conversionDisplay.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
   }
   
   
   
   /**
    * Sets the grid bag layout.
    * ESCA-JAVA0076:
    */   
   private void gridBagConstraints()
   {
      // Initialize grid bag layout of GUI.
  
      constraints = new GridBagConstraints();
      
      constraints.fill = GridBagConstraints.HORIZONTAL;
      
      
      
      // Grid bag constraints 1.
      
      constraints.insets = new Insets(10,0,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 0;
      
      
      contentPane.add(firstCurrencyBox, constraints);  
      
      
      
      // Grid bag constraints 2.
      
      constraints.insets = new Insets(10,10,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 2; 
      
      constraints.gridy = 0;
      
      
      contentPane.add(firstCountryDisplay, constraints);  
      
      
      
      // Grid bag constraints 3.
      
      constraints.insets = new Insets(10,0,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 1;
      
      
      contentPane.add(secondCurrencyBox, constraints);  
 
      
      
      // Grid bag constraints 4.
      
      constraints.insets = new Insets(10,10,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 2; 
      
      constraints.gridy = 1;
      
      
      contentPane.add(secondCountryDisplay, constraints);  
      
      
      
      // Grid bag constraints 5.
      
      constraints.insets = new Insets(30,0,0,0);
      
      
      constraints.ipadx = 350; 
      
      constraints.ipady = 1;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 3;
      
      
      contentPane.add(amountField, constraints);  
      
      
      
      // Grid bag constraints 6.
      
      constraints.insets = new Insets(30,0,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 50;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 4;
      
      
      contentPane.add(conversionDisplay, constraints);  
      
      
      
      // Grid bag constraints 7.
      
      constraints.anchor = GridBagConstraints.PAGE_END;
      
      constraints.insets = new Insets(10,10,10,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 1; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 5;
      
      
      contentPane.add(buttonPanel, constraints);
   }
   
   
   //*************************************** END INITIALISATION CODE ***************************************//
   
   
   //************************************* START INTERFACE EVENT CODE **************************************//
   
   
   private class ClearButtonListener   
   implements ActionListener
   {

      public void actionPerformed(ActionEvent event)
      {
         amountField.setText(null);
         
         conversionDisplay.setText(null);
      }
   }
   
   
   
   private class FirstCurrencyListener   
   implements ActionListener
   {

      public void actionPerformed(ActionEvent event)
      {
         firstCountryDisplay.setText
         (
            currency.returnCountryName((String)firstCurrencyBox.getSelectedItem())
         );
      }
   }
   
   
   
   private class SecondCurrencyListener   
   implements ActionListener
   {

      public void actionPerformed(ActionEvent event)
      {
         secondCountryDisplay.setText
         (
            currency.returnCountryName((String)secondCurrencyBox.getSelectedItem())
         );
      }
   }
   
   
   
   private class RunButtonListener
   implements ActionListener
   {
  
      /** 
       * Listener that starts the counter.
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void actionPerformed(ActionEvent event)
      { 
         
      
         setFirstCurrency((String)firstCurrencyBox.getSelectedItem());
         
         setSecondCurrency((String)secondCurrencyBox.getSelectedItem());
         
         
         if (validation.numberValidation(amountField.getText()))
         {
            setConvertAmount(amountField.getText()); 
    
         
            try 
            {
               currency.setFeed
               (
                  currency.returnCountryCode(getFirstCurrency()), 
    
                  currency.returnCountryCode(getSecondCurrency()), 
     
                  getConvertAmount()
               );
            
            
               conversionDisplay.setText(returnDisplayContent());
            }
         
            catch (Exception e)
            {System.out.println(Tools.thisPathAndLine() + e + "\n");}
         
         }
      }
   }
   

   //************************************** END INTERFACE EVENT CODE ***************************************//
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   
   
   /**
    * Sets the visibility of the frame.
    */
   public void setVisibility()
   {
   
      if (isVisible) {frame.setVisible(true);}
      
      else {frame.setVisible(false);} 
   }
   
   
   
   /**
    * Displays or Hides contents of GUI.
    * @param shown
    */
   public void setContentDisplay(boolean shown)
   {
      contentPane.setVisible(shown);
   }
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
   
   
   //**************************************** START PRIVATE METHODS ****************************************//
   
   
   /**
    * Returns the display content.
    * @throws IOException 
    * @return
    */
   private String returnDisplayContent() throws IOException
   {
      String content = Tools.returnNewline(1) + Tools.returnSpace(5) + 
            
      getFirstCurrency() + " - " + getConvertAmount() + Tools.returnNewline(2) + 
            
      Tools.returnSpace(5) + getSecondCurrency() + " - " + currency.returnDataFeed() + 

      
      Tools.returnNewline(4) + Tools.returnSpace(5) + "Currency code for " + 
            
      currency.returnCountryName(getFirstCurrency()) + " - " + 
            
      currency.returnCountryCode(getFirstCurrency()) +
            
            
      Tools.returnNewline(2) + Tools.returnSpace(5) + "Currency code for " + 
            
      currency.returnCountryName(getSecondCurrency()) + " - " + 
            
      currency.returnCountryCode(getSecondCurrency());
      
      
      return content;
   }
   
   
   
   /**
    * Sets the first currency.
    * @param value
    */
   private void setFirstCurrency(String value)
   {
      firstCurrency = value;
   }
   
   
   
   /**
    * Sets the second currency.
    * @param value
    */
   private void setSecondCurrency(String value)
   {
      secondCurrency = value;
   }
   
   
   
   /**
    * Sets the conversion amount.
    * @param value
    */
   private void setConvertAmount(String value)
   {
      if (!value.equals("")) 
      { 
         convertAmount = value;
      }
   }
   
   
   
   /**
    * Returns the first currency.
    * @return
    */
   private String getFirstCurrency()
   {
      return firstCurrency;
   }
   
   
   
   /**
    * Returns the second currency.
    * @return
    */
   private String getSecondCurrency()
   {
      return secondCurrency;
   }
   
   
   
   /**
    * Returns the conversion amount.
    * @return
    */
   private String getConvertAmount()
   {
      return convertAmount;
   }
   
   
   
   /**
    * Window resize event.
    */
   private void includeResizeEvent()
   {
      this.addComponentListener 
      (
         new ComponentAdapter() 
         {
            public void componentResized(ComponentEvent e) 
            {
               appletDisplayHandler();
            }
          
         }
      ); 
   }
   
   
   
   /**
    * Controls the JApplet component visibility. 
    * ESCA-JAVA0266:
    */
   private void appletDisplayHandler()
   {
      final String ZOOM_WARNING = 
   
      "It appears that web pages in your browser are zoomed out. \n\n" +
      "In order to display the applet, your browser needs to be set \n" +
      "at its default zoom level of 100% or greater. \n\n" +
      "Sorry about that.";
   
   
      if ((getWidth() < FRAME_SIZE_X) || (getHeight() < FRAME_SIZE_Y))
      {
         contentPane.setVisible(false);  
      
         
         if (!zoomWarningShown) 
         {
            Message.showMessage(ZOOM_WARNING, "Browser zoom setting", "warning");
            
            zoomWarningShown = true;
         } 
      }
       
      else 
      {
         contentPane.setVisible(true);
      } 
   }
   
   
   //***************************************** END PRIVATE METHODS *****************************************//

   
   //********************************************** START MAIN *********************************************//
   
   
   /**
    * Main
    * @param a
    */
   public static void main(String[] a)
   {
      isApplication = true;
      
      isVisible = true;
      
      new CurrencyConverterGui();
   }
   
   
   //*********************************************** END MAIN **********************************************//

   
   /**
    * ESCA-JAVA0007:
    */
   public JFrame frame;

   private JButton run, clear;
   private JPanel buttonPanel;
   private Container contentPane;
   private Validation validation;
   private JTextField amountField;
   private boolean zoomWarningShown;
   private CurrencyConverter currency;
   private GridBagConstraints constraints;
   private String convertAmount, firstCurrency, secondCurrency;
   private JComboBox<String> firstCurrencyBox, secondCurrencyBox;
   private JTextArea firstCountryDisplay, secondCountryDisplay, conversionDisplay;
}


//********************************************** END CLASS **********************************************//