/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package required;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;


//********************************************* START CLASS *********************************************//


/**
 *  This class is used to convert between currencies.
 *
 *  @author Alexander Wait
 *  @version 2012-05-05
 */
public class CurrencyConverter
{
   private static final String DATA_FEED_KEY = "zwh3O-ExF36-EztRG";
   
   
   private static final int COUNTRY_NAME_INDEX = 0;
   private static final int CURRENCY_NAME_INDEX = 1;
   private static final int COUNTRY_CODE_INDEX = 2;
   
   
   /**
    * Application Constructor.
    * ESCA-JAVA0057:
    */
   public CurrencyConverter() {}
   
   
   //**************************************** START PUBLIC METHODS *****************************************//

   
   /**
    * Sets the data feed.
    * @param fromCurrency
    * @param toCurrency
    * @param amount
    * @throws MalformedURLException
    */
   public void setFeed(String fromCurrency, String toCurrency, String amount) 
   throws MalformedURLException
   {
      String path = "http://www.exchangerate-api.com/" + fromCurrency + "/" + 
   
      toCurrency + "/" + amount + "?k=" + DATA_FEED_KEY;
   
   
      convert = new URL(path);
   }
   
   
   
   /**
    * Returns the data feed.
    * @return
    * @throws IOException
    */
   public String returnDataFeed() throws IOException
   {
      BufferedReader in = new BufferedReader(new InputStreamReader(convert.openStream()));
  
      String answer = in.readLine(); in.close();
      
      
      return answer;
   }
   
   
   
   /**
    * Returns the currency array.
    * @return
    */
   public String[] returnCurrencyName()
   {
      String[] currencyName = new String[currencies.length];
      
      
      for (int i = 0; i < currencies.length; i ++)
      {
         String[] parts = currencies[i].split(",");
         
         currencyName[i] = parts[CURRENCY_NAME_INDEX]; 
      }

      
      return currencyName;
   }
   
   
   
   /**
    * Returns the country name given the currency.
    * @param currencyName
    * @return
    */
   public String returnCountryName(String currencyName)
   {
      String countryName = null;
   
      
      for (String currencie : currencies) 
      {
         String[] parts = currencie.split(",");
         
         if(currencyName.equals(parts[CURRENCY_NAME_INDEX]))
         {
            countryName = parts[COUNTRY_NAME_INDEX];
         }
      }
      
      return countryName;
   }
   
   
   
   /**
    * Returns the country code given the currency.
    * @param currencyName
    * @return
    */
   public String returnCountryCode(String currencyName)
   {
      String countryCode = null;
   
      
      for (String currencie : currencies) 
      {
         String[] parts = currencie.split(",");
         
         if(currencyName.equals(parts[CURRENCY_NAME_INDEX]))
         {
            countryCode = parts[COUNTRY_CODE_INDEX];
         }
      }
      
      return countryCode;
   }
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
   
   
   private URL convert;
   
   private String[] currencies =
   {
      "Argentina,Argentine Peso,ARS","Australia,Australian Dollar,AUD",
      "Bahamas,Bahamian Dollar,BSD","Bahrain,Bahraini Dinar,BHD",
      "Barbados,Barbados Dollar,BBD","Benin,Cfa Franc Bceao,XOF",
      "Brazil,Brazilian Real,BRL","Cameroon,Cfa Franc Beac,XAF",
      "Canada,Canadian Dollar,CAD","Chile,Chilean Peso,CLP",
      "China,Chinese Yuan,CNY","Croatian,Croatia Kuna,HRK",
      "Czech Republic,Czech Koruna,CZK","Denmark,Danish Krone,DKK",
      "East Caribbean,East Caribbean Dollar,XCD","Egypt,Egyptian Pound,EGP",
      "Estonia,Estonian Kroon,EEK","Eurozone,Euro,EUR",
      "Fiji,Fiji Dollar,FJD","Hong Kong,Hong Kong Dollar,HKD",
      "Hungary,Hungarian Forint,HUF","Iceland,Icelandic Krona,ISK",
      "India,Indian Rupee,INR","Indonesia,Indonesian Rupiah,IDR",
      "Israel,Israeli New Shekel,ILS","Jamaica,Jamaican Dollar,JMD",
      "Japan,Japanese Yen,JPY","Kenya,Kenyan Shilling,KES",
      "Korea,Korean Won,KRW","Latvian Lats,Latvia,LVL",
      "Lithuania,Lithuanian Litas,LTL","Malasia,Malasian Ringgit,MYR",
      "Mexico,Mexican Pesos,MXN","Morocco,Moroccan Dirham,MAD",
      "Netherlands Antilles,Netherlands Antillian Guilder,ANG",
      "New Zealand,New Zealand Dollar,NZD","Norway,Norwegian Krone,NOK",
      "Oman,Omani Rial,OMR","Pakistan,Pakistani Rupee,PKR",
      "Panama,Panamanian Balboa,PAB","Peru,Peruvian Nuevo Sol,PEN",
      "Philippines,Philippine Peso,PHP","Poland,Polish Zloty,PLN",
      "Qatar,Qatari Rial,QAR","Romania,New Romanian Leu,RON",
      "Russia,Russian Rouble,RUB","Saudi Arabia,Saudi Riyal,SAR",
      "Serbia,Serbian Dinar,RSD","Singapore,Singapore Dollar,SGD",
      "South Africa,South African Rand,ZAR","Sri Lanka,Sri Lanka Rupee,LKR",
      "Sweden,Swedish Krona,SEK","Switzerland,Swiss Franc,CHF",
      "Taiwan,Taiwan New Dollar,TWD","Thailand,Thai Baht,THB",
      "Trinidad and Tobago,Trinidad And Tobago Dollar,TTD",
      "Turkey,New Turkish Lira,TRY","United Arab Emirates,Uae Dirham,AED",
      "United Kingdom,Pound Sterling,GBP","United States,US Dollar,USD",
      "Venezuela,Venezuelan Bolivar Fuerte,VEF","Viet Nam,Vietnamese Dong,VND"
   };
}


//********************************************** END CLASS **********************************************//